#/************************************************************
#**+----------------------------------------------------------------------------------+**
#**|                            ****                                                  |**
#**|                            ****                                                  |**
#**|                            ******o***                                            |**
#**|                      ********_///_****                                           |**
#**|                      ***** /_//_/ ****                                           |**
#**|                       ** ** (__/ ****                                            |**
#**|                           *********                                              |**
#**|                            ****                                                  |**
#**|                            ***                                                   |**
#**|                                                                                  |**
#**|         Copyright (c) 1998-2006 Texas Instruments Incorporated                   |**
#**|                        ALL RIGHTS RESERVED                                       |**
#**|                                                                                  |**
#**| Permission is hereby granted to licensees of Texas Instruments                   |**
#**| Incorporated (TI) products to use this computer program for the sole             |**
#**| purpose of implementing a licensee product based on TI products.                 |**
#**| No other rights to reproduce, use, or disseminate this computer                  |**
#**| program, whether in part or in whole, are granted.                               |**
#**|                                                                                  |**
#**| TI makes no representation or warranties with respect to the                     |**
#**| performance of this computer program, and specifically disclaims                 |**
#**| any responsibility for any damages, special or consequential,                    |**
#**| connected with the use of this program.                                          |**
#**|                                                                                  |**
#**+----------------------------------------------------------------------------------+**
#**  script to test basic nat/firewall functionality in standalone mode
#************************************************************/

#!/bin/sh

if [ "$#" -le 2 -a "$#" -eq "1" -a "$1" -ne "9" ]
then
	echo "$0 1 <wan i/f> <wan IP> enable firewall  & NAT"
	echo "$0 2 <wan i/f> <wan IP> <lan DMZ host> firewall, NAT & DMZ"
	echo "$0 3 <wan i/f> <wan IP> <lan host> firewall, NAT & port forwarding"
	echo "$0 4 <wan i/f> <lan i/f> <lan host> block/filter ftp traffic from given host to wan"
	echo "$0 9 flush all rules in filter, nat and mangle table"
	exit
fi

WAN_INTERFACE=$2
IP=$3

echo "WAN_INTERFACE = $WAN_INTERFACE"

if [ "$1" -eq "9" ]
then
	echo "Flushing rules in filter, nat and mangle table"
	iptables -t filter -F	
	iptables -t nat -F	
	iptables -t mangle -F	
fi

if [ "$1" -eq "1" ]
then
	echo "enabling firewall and NAT, allowing only pings to box"
	# to box
	iptables -A INPUT -i $WAN_INTERFACE -j DROP 
	# to LAN side
	iptables -A FORWARD -i $WAN_INTERFACE -j DROP 

# enable established TCP connections
	# from  box
	iptables -I INPUT -i $WAN_INTERFACE -m state --state ESTABLISHED,RELATED -j ACCEPT
	# from LAN side
	iptables -I FORWARD -i $WAN_INTERFACE -m state --state ESTABLISHED,RELATED -j ACCEPT

# enable NAT
	iptables -A POSTROUTING -t nat -o $WAN_INTERFACE -j MASQUERADE

# allow icmp echo request
	iptables -I INPUT -i $WAN_INTERFACE -p icmp --icmp-type echo-request -m state --state NEW -j ACCEPT	
	
fi

if [ "$1" -eq "2" ]
then

	if [ "$#" -ne "4" ]
	then
		echo "Not enough arguments"
		exit
	fi

	DMZ_HOST=$4
	echo "WAN_IP = $IP"
	echo "enabling Firewall, nat and DMZ to host $DMZ_HOST"

# drop packets to INPUT chain from WAN

	iptables -A INPUT -i $WAN_INTERFACE -j DROP 

# enable established TCP connections
	# to  box
	iptables -I INPUT -i $WAN_INTERFACE -m state --state ESTABLISHED,RELATED -j ACCEPT
	# to LAN side
	iptables -I FORWARD -i $WAN_INTERFACE -m state --state ESTABLISHED,RELATED -j ACCEPT

# enable NAT
	iptables -A POSTROUTING -t nat -o $WAN_INTERFACE -j MASQUERADE

# enable DMZ

    
	# direct all packets not to our IP to INPUT chain
	iptables -t nat -A PREROUTING -i $WAN_INTERFACE -d  $IP -j DNAT --to $DMZ_HOST
	# accept packets to DMS HOST at filter table
	iptables -A FORWARD -i $WAN_INTERFACE -p ALL -d $DMZ_HOST -j ACCEPT
fi

if [ "$1" -eq "3" ]
then

	if [ "$#" -ne "4" ]
	then
		echo "Not enough arguments"
		exit
	fi

	LAN_HOST=$4
	echo "enabling Firewall, nat and Port forwarding (ftp) to host $LAN_HOST"


	iptables -I INPUT -i $WAN_INTERFACE -m state --state ESTABLISHED,RELATED -j ACCEPT
	iptables -A INPUT -i $WAN_INTERFACE -j DROP 
	iptables -I FORWARD -i $WAN_INTERFACE -m state --state ESTABLISHED,RELATED -j ACCEPT

# Port forwarding rule for ftp

# ftp
	iptables -A FORWARD -i $WAN_INTERFACE -p tcp --dport 21:21 -d $LAN_HOST -j ACCEPT 
	iptables -t nat -A PREROUTING -i $WAN_INTERFACE -p tcp --dport 21:21 -j DNAT --to $LAN_HOST:21
# web
	iptables -A FORWARD -i $WAN_INTERFACE -p tcp --dport 80:80 -d $LAN_HOST -j ACCEPT 
	iptables -t nat -A PREROUTING -i $WAN_INTERFACE -p tcp --dport 80:80 -j DNAT --to $LAN_HOST:80
	iptables -A FORWARD -i $WAN_INTERFACE -p tcp --dport 443:443 -d $LAN_HOST -j ACCEPT 
	iptables -t nat -A PREROUTING -i $WAN_INTERFACE -p tcp --dport 443:443 -j DNAT --to $LAN_HOST:443

# telnet 
	iptables -A FORWARD -i $WAN_INTERFACE -p tcp --dport 23:23 -d $LAN_HOST -j ACCEPT 
	iptables -t nat -A PREROUTING -i $WAN_INTERFACE -p tcp --dport 23:23 -j DNAT --to $LAN_HOST:23

# smtp
	iptables -A FORWARD -i $WAN_INTERFACE -p tcp --dport 25:25 -d $LAN_HOST -j ACCEPT 
	iptables -t nat -A PREROUTING -i $WAN_INTERFACE -p tcp --dport 25:25 -j DNAT --to $LAN_HOST:25

# DNS
	iptables -A FORWARD -i $WAN_INTERFACE -p udp --dport 53:53 -d $LAN_HOST -j ACCEPT 
	iptables -t nat -A PREROUTING -i $WAN_INTERFACE -p udp --dport 53:53 -j DNAT --to $LAN_HOST:53

	iptables -A FORWARD -i $WAN_INTERFACE -j DROP 


# enable NAT
	iptables -A POSTROUTING -t nat -o $WAN_INTERFACE -j MASQUERADE

# allow ping
	iptables -I INPUT -i $WAN_INTERFACE -p icmp --icmp-type echo-request -m state --state NEW -j ACCEPT	

fi

if [ "$1" -eq "4" ]
then

	if [ "$#" -ne "4" ]
	then
		echo "Not enough arguments"
		exit
	fi

	LAN_HOST=$4
	LAN_INTERFACE=$3
	echo "enabling Firewall, nat and blocking ftp traffic from host $LAN_HOST at interface $LAN_INTERFACE"


	iptables -A INPUT -i $WAN_INTERFACE -j DROP 
	iptables -I FORWARD -i $WAN_INTERFACE -m state --state ESTABLISHED,RELATED -j ACCEPT

# IP filtering of ftp traffic from a given host 

	iptables -A FORWARD -i $LAN_INTERFACE -p tcp --dport 21:21 -s $LAN_HOST -j DROP 
	iptables -A FORWARD -i $WAN_INTERFACE -j DROP 


# enable NAT
	iptables -A POSTROUTING -t nat -o $WAN_INTERFACE -j MASQUERADE

# allow ping
	iptables -I INPUT -i $WAN_INTERFACE -p icmp --icmp-type echo-request -m state --state NEW -j ACCEPT	

fi



	iptables -L -vn
	iptables -t nat -L -vn
	iptables -t mangle -L -vn
# rule to drop packets by default from WAN
